/*--------------------------------------------------------------*/
/* CONTROL SOLDADURA RESISTIVA POR PUNTOS                       */
/*--------------------------------------------------------------*/
/*                                                              */
/* FECHA: MAYO DE 2016                                          */
/* Grado en ingeniería eléctronica industrial y automática      */
/* Universidad de Valladolid                                    */
/*                                                              */
/*AUTOR: DIEGO RAMOS SÁNCHEZ                                    */
/*diegoramossanchez@gmail.com                                   */
/*--------------------------------------------------------------*/

//LIBRERIAS
#include <LiquidCrystal.h>
#include <Keypad.h>
#include <TimerOne.h>
#include <TimerThree.h>
#include <MsTimer2.h>


//DECLARACION CONSTANTES

//salidas
#define PIN_DISPARO 12
#define PIN_VALVULA_01 11
#define PIN_VALVULA_02 10
#define PIN_SOBRETEMP_TRANS 28
#define PIN_SOBRETEMP_TIRIS 26
#define PIN_FALTA_AIRE 24

//entradas
#define PIN_PASOXCERO 3 //interrupcion
#define PIN_SEN_CORRIENTE A11
#define PIN_SEN_TEMP_TRANS 19 //interrupcion
#define PIN_SEN_TEMP_TIRIS 18 //interrupcion
#define PIN_SEN_PRESION 21 //interrupcion
#define PIN_EMERGENCIA 2 //interrupcion
#define PIN_MODO 17 //AUTO O MANUAL
#define PIN_SOLDADURA 30 //Soldadura si o no
#define PIN_PEDAL 20 //interrupcion


//DECLARACIÓN VARIABLES
int alpha = 180; //angulo de disparo tiristores
int tiempo_alpha = 0; //tiempo hasta el disparo tiristores
int auto_manual = 99; //valor 0 auto, valor 1 manual
int pedal = 0; //pedal de accionamiento 0 NO / 1 SI
int soldadura_ext = 0; //0 no soldadura, 1 soldadura
int soldadura = 0; //periodo de paso de corriente 0 no, 1 si
int emergencia = 0; //0 no emergencia, 1 emergencia activa
int estado_presion = 0; //estado de la presión (0, aproximación; 1,apriete)
int estado_soldar = 0; //estado de la corriente (0, no corriente; 1,corriente)
char tecla = NO_KEY; //última tecla presionada en el matricial
char programa_elegido; //almacena el programa de soldadura elegido


//parametros de soldadura
int tiempo_inicio_soldadura; //tiempo de aplicación de corriente
int tiempo_soldadura; //tiempo de aplicación de corriente
int tiempo_aproximacion; //tiempo de aproximación (válvula 1)
int tiempo_presion; //tiempo de presión (2 valvulas abiertas)
int corriente_soldadura; //se ajusta de 0 a 1024
int corriente_real; //valor de la corriente real

//valores sensores
int sens_corriente = 0;
int sens_presion = 0;

//inicializacion LCD
LiquidCrystal lcd(23, 25, 27, 29, 33, 35);

//TECLADO MATRICIAL
const byte FILAS = 4;
const byte COLUMNAS = 3;
char keys[FILAS][COLUMNAS] = {
  {'1', '2', '3'},
  {'4', '5', '6'},
  {'7', '8', '9'},
  {'*', '0', '#'}
};
byte FILAS_Pins[FILAS] = {41, 43, 45, 47};
byte COLUMNAS_Pins[COLUMNAS] = {49, 51, 53};

Keypad keypad = Keypad( makeKeymap(keys), FILAS_Pins, COLUMNAS_Pins, FILAS, COLUMNAS);


void setup()
{
  pinMode(PIN_DISPARO, OUTPUT);
  pinMode(PIN_VALVULA_01, OUTPUT);
  pinMode(PIN_VALVULA_02, OUTPUT);
  pinMode(PIN_SOBRETEMP_TRANS, OUTPUT);
  pinMode(PIN_SOBRETEMP_TIRIS, OUTPUT);
  pinMode(PIN_FALTA_AIRE, OUTPUT);

  pinMode(PIN_PASOXCERO, INPUT);
  pinMode(PIN_SEN_CORRIENTE, INPUT);
  pinMode(PIN_SEN_TEMP_TRANS, INPUT);
  pinMode(PIN_SEN_TEMP_TIRIS, INPUT);
  pinMode(PIN_SEN_PRESION, INPUT);
  pinMode(PIN_EMERGENCIA, INPUT);
  pinMode(PIN_MODO, INPUT);
  pinMode(PIN_SOLDADURA, INPUT);
  pinMode(PIN_PEDAL, INPUT);

  Serial.begin(9600);
  lcd.begin(16, 2);
  lcd.print("EQUIPO DE SOLDADURA");
  lcd.setCursor(0, 1);
  lcd.print("CARGANDO...");

  //  INTERRUPCIONES
  attachInterrupt(digitalPinToInterrupt(PIN_EMERGENCIA), paro_emergencia, LOW); //interrupción al pulsar la seta de emergencia
  attachInterrupt(digitalPinToInterrupt(PIN_SEN_TEMP_TIRIS), sobretemperatura_tiristores, HIGH); //interrupción calentamiento tiristores
  attachInterrupt(digitalPinToInterrupt(PIN_SEN_TEMP_TRANS), sobretemperatura_transformador, HIGH); //interrupción calentamiento transformador
  attachInterrupt(digitalPinToInterrupt(PIN_PEDAL), pisar_pedal, CHANGE); //interrupción por cambio de estado en pedal
  attachInterrupt(digitalPinToInterrupt(PIN_PASOXCERO), paso_x_cero, RISING); //interrupción paso por cero
  attachInterrupt(digitalPinToInterrupt(PIN_SEN_PRESION), falta_aire, LOW); //interrupción no hay aire comprimido
  Timer1.initialize();
  Timer1.stop();
  Timer3.initialize();
  Timer3.stop();
  Timer1.attachInterrupt(tren_disparo); //cuando pasa el tiempo_alpha se disparan tiristores
  Timer3.attachInterrupt(control_electrovalvulas);

  Serial.begin(9600); //Inicializa puerto serie
}//fin setup

void loop()
{
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("SELECCIONA MODO");
  lcd.setCursor(1, 1);
  lcd.print("(AUTO/MANUAL)");
  auto_manual = digitalRead(PIN_MODO);
  lcd.clear();
  while (auto_manual == 1) //modo manual
  {
    lcd.setCursor(2, 0);
    lcd.print("MODO  MANUAL");
    lcd.setCursor(1, 1);
    lcd.print("NO DISPONIBLE");
    auto_manual = digitalRead(PIN_MODO);
    if (auto_manual == 0) break;
  }

  while (auto_manual == 0) //modo automático
  {
    lcd.clear();
    while (tecla != '*') {
      lcd.setCursor(0, 0);
      lcd.print("MODO  AUTOMATICO");
      lcd.setCursor(0, 1);
      lcd.print("pulse *");
      tecla = keypad.getKey();
      auto_manual = digitalRead(PIN_MODO);
      if (auto_manual == 1) break;
    }
    lcd.clear();
    while (tecla != '#') {
      lcd.setCursor(0, 0);
      lcd.print("PROGRAMA");
      lcd.setCursor(11, 0);
      lcd.print(programa_elegido);
      lcd.setCursor(0, 1);
      lcd.print("# para cambiar");
      tecla = keypad.getKey();
      auto_manual = digitalRead(PIN_MODO);
      if (auto_manual == 1) break;
    }
    selecciona_programa();
    parametros_soldadura_automatica();
    auto_manual = digitalRead(PIN_MODO);
  }

}//fin loop


/***************************************************************/

void tren_disparo()
{
  Serial.println("DISPARO");
  Timer1.stop();
  for (int x = 0; x <= 20; x++) //duración de tren disparos
  {
    digitalWrite(PIN_DISPARO, HIGH);
    delayMicroseconds(50);
    digitalWrite(PIN_DISPARO, LOW);
    delayMicroseconds(50);
  }
}//fin tren_disparo

/***************************************************************/

void calculo_tiempo_alpha()
{
  sens_corriente = analogRead(PIN_SEN_CORRIENTE); 
  corriente_real = map(sens_corriente, 512, 1024, 0, 1024);//VER VALORES QUE MIDE EL SENSOR PARA AJUSTAR EL MAPEO

  if (corriente_real < (corriente_soldadura - 50))
  {
    alpha = alpha + 10;
  } else if (corriente_real < (corriente_soldadura))
  {
    alpha = alpha + 3;
  } else if (corriente_real > (corriente_soldadura))
  {
    alpha = alpha - 3;
  } else if (corriente_real > (corriente_soldadura + 50))
  {
    alpha = alpha - 10;
  }
  tiempo_alpha = map(alpha, 0, 180, 0, 10000); //pasa de grados a microsegundos con f=50Hz
}//FIN calculo_tiempo_alpha

/***************************************************************/

void control_electrovalvulas() {
  Timer3.stop();
  if (estado_presion == 0) {
    digitalWrite (PIN_VALVULA_01, HIGH);//activa electroválvula de aproximación
    digitalWrite (PIN_VALVULA_02, HIGH);//activa electroválvula de presión
    estado_presion = 1;
    Timer3.setPeriod(tiempo_presion);
  } else if (estado_presion == 1) {
    digitalWrite (PIN_VALVULA_01, LOW);//activa electroválvula de aproximación
    digitalWrite (PIN_VALVULA_02, LOW);//activa electroválvula de presión
    estado_presion = 0;
  }
}//FIN control_electrovalvulas

/***************************************************************/

void soldar() {
  MsTimer2::stop();
  if (estado_soldar == 0) {
    estado_soldar = 1;
    soldadura = 1;
    MsTimer2::set(tiempo_soldadura, soldar);
  } else if  (estado_soldar == 1) {
    soldadura = 0;
    estado_soldar = 0;
  }
}//FIN soldar

